import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:device_info/device_info.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:http/http.dart' as http;
import 'package:package_info/package_info.dart';

class AppCenter {
  static const MethodChannel _channel = MethodChannel('app_center_diagnostics_and_analytics');
  static String? appSecretAndroid;
  static String? appSecretIOS;
  static DateTime initTimeStamp = DateTime.now().toUtc();

  static Future<void> startAsync({
    required String appSecretAndroid,
    required String appSecretIOS,
    enableAnalytics = true,
    enableCrashes = true,
  }) async {
    AppCenter.appSecretAndroid = appSecretAndroid;
    AppCenter.appSecretIOS = appSecretIOS;
    AppCenter.initTimeStamp = DateTime.now().toUtc();

    String appSecret;

    if (Platform.isAndroid) {
      appSecret = appSecretAndroid;
    } else if (Platform.isIOS) {
      appSecret = appSecretIOS;
    } else {
      return;
    }

    assert(appSecret.isNotEmpty, 'AppSecret cannot be empty');

    WidgetsFlutterBinding.ensureInitialized();

    await configureAnalyticsAsync(enabled: enableAnalytics);
    await configureCrashesAsync(enabled: enableCrashes);

    return _channel.invokeMethod('start', <String, dynamic>{
      'secret': appSecret,
    });
  }

  /// Get appcenter installation id
  static Future<String> getInstallIdAsync() async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return '';
    }

    return _channel.invokeMethod('getInstallId').then((r) => r as String);
  }

  /// Enable or disable analytics
  static Future<void> configureAnalyticsAsync({required enabled}) async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return;
    }

    return _channel.invokeMethod('configureAnalytics', enabled);
  }

  /// Check whether analytics is enalbed
  static Future<bool?> isAnalyticsEnabledAsync() async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return null;
    }

    return _channel.invokeMethod('isAnalyticsEnabled');
  }

  /// Track events
  static Future<void> trackEventAsync(String name, [Map<String, String>? properties]) async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return;
    }

    return _channel.invokeMethod('trackEvent', <String, dynamic>{
      'name': name,
      'properties': properties ?? <String, String>{},
    });
  }

  /// Enable or disable appcenter crash reports
  static Future<void> configureCrashesAsync({required enabled}) async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return;
    }

    return _channel.invokeMethod('configureCrashes', enabled);
  }

  /// Check whether crashes is enabled
  static Future<bool?> isCrashesEnabledAsync() async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return null;
    }

    return _channel.invokeMethod('isCrashesEnabled');
  }

  /// Track events
  static Future trackCrashAsync({
    Exception? exception,
    Error? error,
    StackTrace? stackTrace,
    Map<String, String>? properties,
  }) async {
    if (!Platform.isAndroid && !Platform.isIOS) {
      return;
    }

    assert(exception != null || error != null, 'Either exception or error needs to be non null');
    assert(exception == null || error == null, 'Do not send both error and exception, split into to trackCrash calls');

    if (Platform.isIOS) {
      return _trackCrashAsyncIos(
        exception: exception,
        error: error,
        stackTrace: stackTrace,
        properties: properties,
      );
    } else if (Platform.isAndroid) {
      return _channel.invokeMethod('trackCrash', <String, dynamic>{
        'throwable': exception ?? error,
        'properties': properties ?? <String, String>{},
        'error': error,
        'exception': exception,
        'stackTrace': stackTrace,
      });
    }
  }

  static Future<void> _trackCrashAsyncIos({
    Exception? exception,
    Error? error,
    StackTrace? stackTrace,
    Map<String, String>? properties,
  }) async {
    final packageInfo = await PackageInfo.fromPlatform();
    final deviceInfo = await DeviceInfoPlugin().iosInfo;

    await http.post(
      Uri(
        scheme: 'https',
        host: 'in.appcenter.ms',
        path: 'logs',
        query: 'Api-Version=1.0.0',
      ),
      headers: <String, String>{
        'Content-Type': 'application/json',
        'App-Secret': AppCenter.appSecretIOS!,
        'Install-ID': deviceInfo.identifierForVendor,
      },
      body: jsonEncode(
        <String, List>{
          'logs': [
            {
              'type': 'managedError',
              'timestamp': DateTime.now().toUtc().toIso8601String(),
              'appLaunchTimestamp': AppCenter.initTimeStamp.toIso8601String(),
              'device': {
                'appVersion': packageInfo.version,
                'appBuild': packageInfo.buildNumber,
                'sdkName': 'appcenter.custom',
                'sdkVersion': '1.9.1',
                'osName': Platform.operatingSystem,
                'osVersion': Platform.operatingSystemVersion,
                'model': deviceInfo.model,
                'locale': Platform.localeName,
                'timeZoneOffset': DateTime.now().timeZoneOffset,
              },
              'userId': deviceInfo.identifierForVendor,
              'exception': {
                'type': (exception ?? error).runtimeType.toString(),
                'message': (exception ?? error).toString(),
                if (stackTrace != null) 'stackTrace': stackTrace.toString(),
              }
            },
          ],
        },
      ),
    );
  }
}
